; RGB-LED mit Tiny12  Lothar Merl 2010
; Sanftes berblenden von 3 LEDs in Rot (portb0), Gruen (portb1) und Blau (portb2)
; zur Erzeugung einer Vielzahl von Mischfarben.
; Anoden der LEDs ueber Widerstand an die Portpins, Kathoden nach GND

.include"tn12def.inc"


;Konstanten:
; b_min, b_max = Brightness_min/max 
; erreicht eine LED beim Abdimmvorgang den Wert von b_min, wird sie auf
; Aufwrtsdimmen umgeschaltet. b_max entsprechend.
.equ h_min=10				; minimale Helligkeit der LEDs
.equ h_max=250				; maximale Helligkeit der LEDs

; Startwerte fure die 3 Farben
; ACHTUNG! Die Werte sollten nicht ausserhalb des oben angegebenen
; Bereiches zwisch h_min und h_max liegen!
.equ start_rot=100
.equ start_gruen=100
.equ start_blau=100


; Timer Startwert/Reloadwert
.equ t_start=247			; sollte 249 nicht uebersteigen, da sonst der Interrupt evtl,
							; zu schnell nacheinander aufgerufen wird. Das kann zu Bereichsueberschreitungen
							; der Helligkeitswerte und zum Flackern der LEDs fuehren.
							; Dieser Wert beeinflut die Gesamtgeschwindigkeit und auch die PWM-Frequenz.
							; Werte unter 240 lassen die LEDs deutlich flimmern. Am besten auf 247 stehen lassen

; Colorspeed (Geschwindigkeit des Farbwechsels) 1=schnell, 255=langsam
.equ colspeed=50

; Bitnummern fuer die Portpins und Dimmer-Richtungen in Variable 'dir':
.equ bit_r=0				; rote LED
.equ bit_g=1				; gruene LED
.equ bit_b=2				; blaue LED
.equ int_merker=7			; wird 1 bei Timerueberlauf

; Geschwindigkeiten fr die Dimmung der einzelnen Farben.
; Es empfiehlt sich, Primzahlen zu waehlen, um das Wiederholungsintervall
; der Mischfarben mglichst gross werden zu lassen. 
.equ s_rot=13
.equ s_gruen=17
.equ s_blau=19

;Register
.def rot=r16				;Helligkeitswert rot
.def gruen=r17				;Helligkeitswert gruen
.def blau=r18				;Helligkeitswert blau 
.def intcount=r19			;zaehlt die Interrupts
.def pwm=r20				;PWM Zaehler
.def dir=r21				;Direction-Register fuer die Dimmer-Richtungen
							;Bits 0-2 = r,g,b  1=heller 0=dunkler
							;ausserdem ist Bit 7 gesetzt, wenn ein Timerueberlauf statt fand
.def cp_r=r22				;Primfaktorzhler rot
.def cp_g=r23				;Primfaktorzhler gruen
.def cp_b=r24				;Primfaktorzhler blau
.def buf=r25				;Puffervariable
;.def int_buf=r26			;Puffervariable fuer Interrupt

.cseg
.org 0						;Reset- und Interrupt-Vektoren AT-Tiny12
 rjmp _RESET				;Reset handler
 rjmp _dummy				;EXT_INT0      ;IRQ0 handler
 rjmp _dummy				;PIN_CHANGE    ;Pin change handler
 rjmp _TIM0_OVF				;Timer0 overflow handler
 rjmp _dummy				;EE_RDY        ;EEPROM Ready handler
 rjmp _dummy				;ANA_COMP      ;Analog Comparator handler
_dummy:						;Interrupt nicht genutzt
 reti


_TIM0_OVF:
 ldi ZH,t_start				;Timer-Startwert setzen (Hatte leider keine Register mehr uebrig, darum ZH)
 out TCNT0,ZH
 dec intcount
 ori dir,1<<int_merker		; Bit 7 der 'dir' Variable setzen
 reti


_RESET:
 ldi buf,7					;Portpins 0,1,2
 out ddrb,buf				;als Ausgang
							
 ldi intcount,100			; Initialisierungen
 ldi rot,start_rot
 ldi gruen,start_gruen
 ldi blau,start_blau
 ldi dir,0

 ldi buf,(1<<cs01)			;Vorteiler auf 8
 out tccr0,buf
 ldi buf,1<<toie0			;Interrupt bei Timer0-berlauf
 out timsk,buf
 ldi buf,t_start			;Timer-Startwert setzen
 out TCNT0,buf

 sei                        ;Interrupts global freigeben [84]


_mainloop:
 cpi intcount,0
 brne _set_leds

 ldi intcount,colspeed		; intcount neu laden

 dec cp_r					; Primfaktorzaehler herunterzaehlen
 brne _gruen				; wenn nicht Null, springe weiter zu gruen
 ldi cp_r,s_rot				; wieder  auf Startwert setzen
 mov buf,dir
 andi buf,1<<bit_r			; pruefen, ob auf oder ab gedimmt wird
 breq _rot_down				; Bit ist Null, also ist richtung 'down'
 cpi rot,h_max				; pruefen, ob rot schon gleich h_max
 breq _rot_max				; Ja, springe zu rot_max
 inc rot					; Rot noch kleiner 255, also rot erhhen
 rjmp _gruen
_rot_max:					; Rot hat Maximalwert erreicht, also
 andi dir,~(1<<bit_r)		; Dimmrichtung von Rot auf 'abwaerts' setzen
 dec rot					; und rot sofort um Eins erniedrigen
 rjmp _gruen				; weiter geht's mit gruen
_rot_down:
 cpi rot,h_min				; ist rot schon auf Minimum?
 breq _rot_min				; ja, also nach rot_min springen und Dimmrichtung auf 'aufwaerts' schalten
 dec rot					; rot ist noch nicht bei Null, also decrementieren
 rjmp _gruen
_rot_min:
 ori dir,1<<bit_r			; Richtungsbit setzen (= 'aufwaerts')
 inc rot 					; und sofort rot erhoehen

_gruen:
 dec cp_g					; Primfaktorzaehler herunterzaehlen
 brne _blau					; wenn nicht Null, springe weiter zu blau
 ldi cp_g,s_gruen			; wieder  auf Startwert setzen
 mov buf,dir
 andi buf,1<<bit_g			; pruefen, ob auf oder ab gedimmt wird
 breq _gruen_down			; Bit ist Null, also ist richtung 'down'
 cpi gruen,h_max			; pruefen, ob gruen schon gleich h_max
 breq _gruen_max			; Ja, springe zu gruen_max
 inc gruen					; Gruen ist noch kleiner 255, also gruen erhhen
 rjmp _blau
_gruen_max:					; Gruen hat Maximalwert erreicht, also
 andi dir,~(1<<bit_g)		; Dimmrichtung von Gruen auf 'abwaerts' setzen
 dec gruen					; und gruen sofort um Eins erniedrigen
 rjmp _blau
_gruen_down:
 cpi gruen,h_min			; ist gruen schon auf Minimum?
 breq _gruen_min			; ja, also nach gruen_min springen und Dimmrichtung auf 'aufwaerts' schalten
 dec gruen					; gruen ist noch nicht bei Null, also decrementieren
 rjmp _blau
_gruen_min:
 ori dir,1<<bit_g			; Richtungsbit setzen (= 'aufwaerts')
 inc gruen 					; und sofort gruen erhoehen

_blau:
 dec cp_b					; Primfaktorzaehler herunterzaehlen
 brne _set_leds				; wenn nicht Null, springe weiter zu set_leds
 ldi cp_b,s_blau			; wieder  auf Startwert setzen
 mov buf,dir
 andi buf,1<<bit_b			; pruefen, ob auf oder ab gedimmt wird
 breq _blau_down			; Bit ist Null, also ist richtung 'down'
 cpi blau,h_max				; pruefen, ob blau schon gleich h_max
 breq _blau_max				; Ja, springe zu blau_max
 inc blau					; Blau ist noch kleiner 255, also blau erhhen
 rjmp _set_leds
_blau_max:					; Blau hat Maximalwert erreicht, also
 andi dir,~(1<<bit_b)		; Dimmrichtung von Blau auf 'abwaerts' setzen
 dec blau					; und blau sofort um Eins erniedrigen
 rjmp _set_leds
_blau_down:
 cpi blau,h_min				; ist blau schon auf Minimum?
 breq _blau_min				; ja, also nach blau_min springen und Dimmrichtung auf 'aufwaerts' schalten
 dec blau					; blau ist noch nicht bei Null, also decrementieren
 rjmp _set_leds
_blau_min:
 ori dir,1<<bit_b			; Richtungsbit setzen (= 'aufwaerts')
 inc blau 					; und sofort blau erhoehen

_set_leds:
 mov buf,dir				; Pruefen, ob es inzwischen einen Interrupt gab (Bit 7 = 1)
 andi buf,1<<int_merker
 breq _mainloop				; Nein, Bit nicht gesetzt, also Kein Interrupt -> zuruech zum Start
 andi dir,~(1<<int_merker)	; Es gab einen Interrupt -> Merkerbit 7 loeschen
 inc pwm					; pwm-Zaehler erhoehen
 brne _pwm_cont				; Wenn pwm gleich Null ist, alle LEDs einschalten
 ldi buf,7					; Bits 0-2 = 1
 out portb, buf				; LEDs an
_pwm_cont:
 cp pwm,rot					; Erreicht der Wert in pwm den jeweiligen Helligkeitswert der LEDs,
 brlo _pwm_1				; wird die betreffende LED ausgeschaltet.
 cbi portb,bit_r
_pwm_1:
 cp pwm,gruen				; pwm und Wert fuer Gruen vergleichen
 brlo _pwm_2				; ist pwm > gruen? Nein, dann nchste LED
 cbi portb,bit_g			; Ja, dann LED aus
_pwm_2:
 cp pwm,blau
 brlo _ende
 cbi portb,bit_b

_ende:
rjmp _mainloop
